package ctbrec.ui.sites.myfreecams;


import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;

import ctbrec.Model;
import ctbrec.sites.mfc.MyFreeCamsClient;
import ctbrec.ui.tabs.PaginatedScheduledService;
import javafx.concurrent.Task;

public class OnlineCamsUpdateService extends PaginatedScheduledService {

    @Override
    protected Task<List<Model>> createTask() {
        return new Task<List<Model>>() {
            @Override
            public List<Model> call() throws IOException {
                var client = MyFreeCamsClient.getInstance();
                var modelsPerPage = 50;
                return client.getModels().stream()
                        .filter(m -> m.getPreview() != null)
                        .filter(m -> m.getStreamUrl() != null)
                        .filter(m -> {
                            try {
                                return m.isOnline();
                            } catch (InterruptedException e) {
                                Thread.currentThread().interrupt();
                                return false;
                            } catch (Exception e) {
                                return false;
                            }
                        })
                        .sorted((m1,m2) -> (int)(m2.getCamScore() - m1.getCamScore()))
                        .skip( (page-1) * (long)modelsPerPage)
                        .limit(modelsPerPage)
                        .collect(Collectors.toList());
            }
        };
    }

}
