package ctbrec.ui.sites.streamate;

import static ctbrec.Model.State.*;
import static ctbrec.io.HttpConstants.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

import ctbrec.Model;
import ctbrec.io.HttpException;
import ctbrec.sites.streamate.Streamate;
import ctbrec.sites.streamate.StreamateHttpClient;
import ctbrec.sites.streamate.StreamateModel;
import ctbrec.ui.tabs.PaginatedScheduledService;
import javafx.concurrent.Task;

public class StreamateUpdateService extends PaginatedScheduledService {

    private static final Logger LOG = LoggerFactory.getLogger(StreamateUpdateService.class);

    private static final int MODELS_PER_PAGE = 48;
    private Streamate streamate;
    private StreamateHttpClient httpClient;
    private String url;

    public StreamateUpdateService(Streamate streamate, String url) {
        this.streamate = streamate;
        this.url = url;
        this.httpClient = (StreamateHttpClient) streamate.getHttpClient();
    }

    @Override
    protected Task<List<Model>> createTask() {
        return new Task<List<Model>>() {
            @Override
            public List<Model> call() throws IOException, SAXException, ParserConfigurationException, XPathExpressionException {
                int from = (page - 1) * MODELS_PER_PAGE;
                String saKey = httpClient.getSaKey();
                String pageUrl = url + "&from=" + from + "&size=" + MODELS_PER_PAGE;
                LOG.debug("Fetching page {}", pageUrl);
                var request = httpClient.newRequestBuilder()
                        .url(pageUrl)
                        .header(ORIGIN, streamate.getBaseUrl())
                        .header("sakey", saKey)
                        .header("platform", "SCP")
                        .header("smtid", UUID.randomUUID().toString() + "G0211569057409")
                        .header("smeid", UUID.randomUUID().toString() + "G0211569057409")
                        .header("smvid", UUID.randomUUID().toString() + "G0211569057409")
                        .build();
                try (var response = httpClient.execute(request)) {
                    if (response.isSuccessful()) {
                        return parseModels(response.body().string());
                    } else {
                        throw new HttpException(response.code(), response.message());
                    }
                }
            }
        };
    }

    private List<Model> parseModels(String content) {
        List<Model> models = new ArrayList<>();
        var json = new JSONObject(content);
        var performers = json.getJSONArray("performers");
        for (var i = 0; i < performers.length(); i++) {
            var p = performers.getJSONObject(i);
            var nickname = p.getString("nickname");
            StreamateModel model = (StreamateModel) streamate.createModel(nickname);
            model.setId(p.getLong("id"));
            model.setPreview("https://cdn.nsimg.net/snap/320x240/" + model.getId() + ".jpg");
            model.setDescription(p.optString("headlineMessage"));
            var online = p.optBoolean("online");
            model.setOnline(online);
            model.setOnlineState(online ? ONLINE : OFFLINE);
            // TODO figure out, what all the states mean
            // liveState {…}
            // exclusiveShow false
            // goldShow true
            // onBreak false
            // partyChat true
            // preGoldShow true
            // privateChat false
            // specialShow false
            if (p.optBoolean("onBreak")) {
                model.setOnlineState(AWAY);
            } else if (p.optBoolean("goldShow") || p.optBoolean("privateChat") || p.optBoolean("exclusiveShow")) {
                model.setOnlineState(PRIVATE);
            }
            models.add(model);
        }
        return models;
    }
}
